# EasyPodcasting Plugin
# Copyright 2006 M-Logic, Inc. All Right Reserved.
# $Id: mt-easypodcasting.pl 49 2013-10-24 02:12:57Z tajima $

package MT::Plugin::EasyPodcasting;

use strict;

use MT::Template::Context;
use base qw( MT::Plugin );

my $PLUGIN_NAME = 'EasyPodcasting';
my $VERSION = '1.30';
 
require MT::Plugin;
my $plugin = MT::Plugin::EasyPodcasting->new({
    name => $PLUGIN_NAME,
    version => $VERSION,
    description => 'EasyPodcasting allows you to easily start Podcasting with Movable Type!',
    doc_link => 'http://labs.m-logic.jp/plugins/mt-easypodcasting/docs/mt-easypodcasting.html',
    author_name => 'M-Logic, Inc.',
    author_link => 'http://labs.m-logic.jp/',
});
MT->add_plugin($plugin);

if (MT->version_number < 4) {
    MT::Template::Context->add_global_filter( EasyPodcasting => \&fltr_Podcastig );
    MT::Template::Context->add_tag( EasyPodcasting => \&hdlr_Podcasting );
    MT::Template::Context->add_tag( EasyPodcastingAtom => \&hdlr_Podcasting_atom );
}

sub init_registry {
    my $plugin = shift;

    return if (MT->version_number < 4);

    $plugin->registry({
        object_types => {
            'asset.mp3'     => 'MT::Asset::MP3',
            'mp3'           => 'MT::Asset::MP3',
        },
        tags => {
            function => {
                'EasyPodcasting' => sub{&hdlr_Podcasting;},
                'EasyPodcastingAtom' => sub{&hdlr_Podcasting_atom;},
            },
            modifier => {
                'EasyPodcasting' => sub{&fltr_Podcasting;},
            },
        },
    });
}

our $regexp = qq(\\[mp3:(https?:\\/\\/?[-_.!~*'()a-zA-Z0-9;\\/?:\\@&=+\$,%#]+?\\.mp3)(:(.+))?\\]);

###----------------------------------------------------------------------###
sub hdlr_Podcasting {
    my ($ctx, $args) = @_;

    return make_enclosure_tag($ctx, $args, '');

}

sub hdlr_Podcasting_atom {
    my ($ctx, $args) = @_;

    return make_enclosure_tag($ctx, $args, 'Atom');
}

sub fltr_Podcasting {
    my ($text, $arg, $ctx) = @_;
    
    my $out_text = $text;

    $out_text =~ s/$regexp/mp3_tag($1, $3, $arg, $ctx)/ige;

    $out_text;
}

sub mp3_tag {
    my ($url, $title, $arg, $ctx) = @_;

    my ($mp3URL ,$mp3Len) = getMp3Info($ctx, $url);

    return '' if $mp3Len == 0;
    
    $title = $mp3URL unless $title;

    my $res;    
    if ($arg eq 'player') {
        $res = mp3_tag_player($ctx, $mp3URL, $mp3Len, $title);
    } elsif ($arg eq 'link') {
        $res = mp3_tag_link($ctx, $mp3URL, $mp3Len, $title);
    } elsif ($arg eq 'player_with_link') {
        $res = mp3_tag_player_with_link($ctx, $mp3URL, $mp3Len, $title);
    } else {
        $res = '';
    }

    $res;
}

sub mp3_tag_link {
    my ($ctx, $url, $len, $title) = @_;

    my $html = <<"EOT";
<div class="mp3_link">
<a href="$url" class="link_mp3" title="$title">$title</a>
</div>
EOT

    $html;
}

sub mp3_tag_player {
    my ($ctx, $url, $len, $title) = @_;

    my $path = MT->instance->static_path;
    $path .= '/' unless $path =~ m!/$!;
    $path .= 'plugins/EasyPodcasting/mp3player.swf';

    my $html = <<"EOT";
<div class="mp3_player">
<object classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" codebase="http://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=8,0,0,0" width="250" height="50" id="mp3player" align="middle">
    <param name="allowScriptAccess" value="sameDomain" />
    <param name="movie" value="$path?mp3url=$url" />
    <param name="quality" value="high" />
    <param name="bgcolor" value="#ffffff" />

    <embed src="$path?mp3url=$url" quality="high" bgcolor="#ffffff" width="250" height="50" name="mp3player" align="middle" allowScriptAccess="sameDomain" type="application/x-shockwave-flash" pluginspage="http://www.macromedia.com/go/getflashplayer" />
</object>
</div>
EOT

    $html;
}

sub mp3_tag_player_with_link {
    my ($ctx, $url, $len, $title) = @_;

    my $html = "<div class=\"mp3\">\n";
    $html .= mp3_tag_player($ctx, $url, $len, $title);
    $html .= mp3_tag_link($ctx, $url, $len, $title);
    $html .= "</div>\n";

    $html;
}

sub getMp3Info {
    my ($ctx, $url) = @_;

    my $mp3URL;
    my $mp3Len = 0;

    my $siteURL = $ctx->stash('blog')->site_url;
    if (index($url, $siteURL) == 0) {
        my $path = substr($url, length($siteURL));
        my $file = File::Spec->catfile( $ctx->stash('blog')->site_path, $path );
        if ( -f $file ) {
            $mp3URL = $url;
            $mp3Len = -s $file;
        }
    }

    if ( $mp3Len == 0 && $url =~ /^https?\:\/\/.*$/i ) {
        my $ua = MT->new_ua;
        $ua->max_size( undef ) if $ua->can('max_size');
        my $request = new HTTP::Request HEAD => $url;
        my $response = $ua->request( $request );
        if ( $response->is_success && $response->content_length) {
            $mp3URL = $url;
            $mp3Len = $response->content_length;
        }
    }
    
    return ($mp3URL, $mp3Len);
}

sub make_enclosure_tag {
    my ($ctx, $args, $type) = @_;

    my $entry = $ctx->stash('entry');
    my $text = '';
    my $out_text = '';

    my $tag = 'MTEasyPodcasting' . $type;
    if ($entry) {
        $text = $entry->text;
        return '' unless $text;
        while ($text =~ m/$regexp/ig) {
            my ($mp3URL ,$mp3Len) = getMp3Info($ctx, $1);
            if ($type eq '') {
                $out_text .= <<"EOT";
<enclosure url="$mp3URL" length="$mp3Len" type="audio/mpeg"/>
EOT
            } elsif ($type eq 'Atom') {
                $out_text .= <<"EOT";
<link rel="enclosure" title="MP3" type="audio/mpeg" href="$mp3URL" length="$mp3Len" />
EOT
            }
        }
    } else {
        return $ctx->error( "You used an <$tag> tag outside of the context of an entry." );
    }
    
    $out_text;
}
